from cpl_core.database.context import DatabaseContextABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_query.extension import List

from bot_api.logging.api_logger import ApiLogger
from bot_api.route.route import Route
from bot_core.service.config_service import ConfigService
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.technician_config_repository_abc import TechnicianConfigRepositoryABC
from bot_data.model.technician_config import TechnicianConfig
from bot_data.model.technician_id_config import TechnicianIdConfig
from bot_data.model.technician_ping_url_config import TechnicianPingUrlConfig
from bot_data.model.user_role_enum import UserRoleEnum
from bot_data.service.technician_config_seeder import TechnicianConfigSeeder
from bot_graphql.abc.query_abc import QueryABC


class TechnicianConfigMutation(QueryABC):
    def __init__(
        self,
        logger: ApiLogger,
        bot: DiscordBotServiceABC,
        servers: ServerRepositoryABC,
        technician_configs: TechnicianConfigRepositoryABC,
        db: DatabaseContextABC,
        config_service: ConfigService,
        tech_seeder: TechnicianConfigSeeder,
    ):
        QueryABC.__init__(self, "TechnicianConfigMutation")

        self._logger = logger
        self._bot = bot
        self._servers = servers
        self._technician_configs = technician_configs
        self._db = db
        self._config_service = config_service
        self._tech_seeder = tech_seeder

        self.set_field("updateTechnicianConfig", self.resolve_update_technician_config)

    def resolve_update_technician_config(self, *_, input: dict):
        if not self._technician_configs.does_technician_config_exists():
            self._bot.loop.create_task(self._tech_seeder.seed())

        technician_config = self._technician_configs.get_technician_config()
        self._can_user_mutate_data(Route.get_user().users[0].server, UserRoleEnum.technician)

        technician_config.help_command_reference_url = (
            input["helpCommandReferenceUrl"]
            if "helpCommandReferenceUrl" in input
            else technician_config.help_command_reference_url
        )
        technician_config.wait_for_restart = (
            input["waitForRestart"] if "waitForRestart" in input else technician_config.wait_for_restart
        )
        technician_config.wait_for_shutdown = (
            input["waitForShutdown"] if "waitForShutdown" in input else technician_config.wait_for_shutdown
        )
        technician_config.cache_max_messages = (
            input["cacheMaxMessages"] if "cacheMaxMessages" in input else technician_config.cache_max_messages
        )
        old_feature_flags = technician_config.feature_flags
        technician_config.feature_flags = (
            dict(zip([x["key"] for x in input["featureFlags"]], [x["value"] for x in input["featureFlags"]]))
            if "featureFlags" in input
            else technician_config.feature_flags
        )
        for old_flag in old_feature_flags:
            if old_flag not in technician_config.feature_flags:
                technician_config.feature_flags[old_flag] = False

        technician_config.ping_urls = (
            List(str, input["pingURLs"]) if "pingURLs" in input else technician_config.ping_urls
        )
        technician_config.technician_ids = (
            List(int).extend([int(x) for x in input["technicianIds"]])
            if "technicianIds" in input
            else technician_config.technician_ids
        )

        self._technician_configs.update_technician_config(technician_config)
        if "pingURLs" in input:
            self._update_ping_urls(technician_config)

        if "technicianIds" in input:
            self._update_technician_ids(technician_config)

        self._db.save_changes()
        self._config_service.reload_technician_config()
        return technician_config

    def _update_ping_urls(self, new_config: TechnicianConfig):
        old_config = self._technician_configs.get_technician_config()
        for url in old_config.ping_urls:
            if url in new_config.ping_urls:
                continue

            self._technician_configs.delete_technician_ping_url_config(TechnicianPingUrlConfig(url))

        for url in new_config.ping_urls:
            if url in old_config.ping_urls:
                continue

            self._technician_configs.add_technician_ping_url_config(TechnicianPingUrlConfig(url))

    def _update_technician_ids(self, new_config: TechnicianConfig):
        old_config = self._technician_configs.get_technician_config()
        for technician_id in old_config.technician_ids:
            if technician_id in new_config.technician_ids:
                continue

            self._technician_configs.delete_technician_id_config(TechnicianIdConfig(technician_id))

        for technician_id in new_config.technician_ids:
            user = self._bot.get_user(technician_id)
            if user is None:
                raise ValueError(f"Invalid technicianId")

        for technician_id in new_config.technician_ids:
            if technician_id in old_config.technician_ids:
                continue

            self._technician_configs.add_technician_id_config(TechnicianIdConfig(technician_id))
